@@ -10,7 +10,7 @@ if [ -z "$1" ] ; then
   echo FCITX sources are now prepared. To build here, run:
   echo " ./configure && make"
 else
-  ./configure $* || exit 4
+  ./configure "$@" || exit 4
   make || exit 5
 fi
 
@@ -30,8 +30,3 @@ echo '  ./configure && make dist'
 echo '  rpmbuild -ts fcitx-*.tar.gz   # build source package'
 echo '  rpmbuild -tb fcitx-*.tar.gz   # build binary package'
 
-echo
-echo 'Build DEBIAN Package like these:'
-echo '  ./configure'
-echo '  vim ./debian/changelog        # add change log'
-echo '  fakeroot dpkg-buildpackage'
