@@ -20,6 +20,7 @@
 #include "MainWindow.h"
 
 #include <stdio.h>
+#include <X11/xpm.h>
 
 #include "IC.h"
 #include "ui.h"
@@ -51,8 +52,8 @@ int             MAINWND_WIDTH = _MAINWND_WIDTH;
 int             iMainWindowX = MAINWND_STARTX;
 int             iMainWindowY = MAINWND_STARTY;
 
-WINDOW_COLOR    mainWindowColor = { NULL, NULL, {0, 255 << 8, 240 << 8, 255 << 8} };
-MESSAGE_COLOR   mainWindowLineColor = { NULL, {0, 220 << 8, 0, 0} };	//ɫ
+WINDOW_COLOR    mainWindowColor = { NULL, NULL, {0, 240 << 8, 255 << 8, 240 << 8} };
+MESSAGE_COLOR   mainWindowLineColor = { NULL, {0, 150 << 8, 220 << 8, 150 << 8} };	//ɫ
 MESSAGE_COLOR   IMNameColor[3] = {	//뷨Ƶɫ
     {NULL, {0, 170 << 8, 170 << 8, 170 << 8}},
     {NULL, {0, 150 << 8, 200 << 8, 150 << 8}},
@@ -80,15 +81,15 @@ XImage         *pVK = NULL;
 
 HIDE_MAINWINDOW hideMainWindow = HM_SHOW;
 
-Bool            bLocked = True;
 Bool            bCompactMainWindow = False;
-Bool            bShowVK = True;
+Bool            bShowVK = False;
 
 extern Display *dpy;
 extern GC       dimGC;
 extern int      i3DEffect;
 extern IC      *CurrentIC;
 extern Bool     bCorner;
+extern Bool     bLocked;
 extern Bool     bChnPunc;
 extern INT8     iIMIndex;
 extern Bool     bUseGBK;
@@ -119,7 +120,7 @@ Bool CreateMainWindow (void)
     attrib.override_redirect = True;
     attribmask = CWOverrideRedirect;
 
-    sprintf (strMainWindowXPMBackColor, ". 	c #%2x%2x%2x", mainWindowColor.backColor.red >> 8, mainWindowColor.backColor.green >> 8, mainWindowColor.backColor.blue >> 8);
+    sprintf (strMainWindowXPMBackColor, ". 	c #%02x%02x%02x", mainWindowColor.backColor.red >> 8, mainWindowColor.backColor.green >> 8, mainWindowColor.backColor.blue >> 8);
 
     if (XAllocColor (dpy, DefaultColormap (dpy, DefaultScreen (dpy)), &(mainWindowColor.backColor)))
 	iBackPixel = mainWindowColor.backColor.pixel;
@@ -139,26 +140,45 @@ Bool CreateMainWindow (void)
 }
 
 void DisplayMainWindow (void)
+{
+#ifdef _DEBUG
+    fprintf (stderr, "DISPLAY MainWindow\n");
+#endif
+
+    XMapRaised (dpy, mainWindow);
+}
+
+void DrawMainWindow (void)
 {
     INT8            iIndex = 0;
     INT16           iPos;
 
+    int             rv;
+    XImage         *mask;
+    XpmAttributes   attrib;
+
 #ifdef _USE_XFT
     char            strTemp[MAX_IM_NAME + 1];
     char           *p1, *p2;
     Bool            bEn;
 #endif
-    iIndex = IS_CLOSED;
-    if (hideMainWindow == HM_SHOW || (hideMainWindow == HM_AUTO && (ConnectIDGetState (connect_id) != IS_CLOSED))) {
-	XMapRaised (dpy, mainWindow);
+    char           *strGBKT;
 
+    iIndex = IS_CLOSED;
+    attrib.valuemask = 0;
+#ifdef _DEBUG
+    fprintf (stderr, "DRAW MainWindow\n");
+#endif
+    if (hideMainWindow == HM_SHOW || (hideMainWindow == HM_AUTO && (ConnectIDGetState (connect_id) != IS_CLOSED))) {
 	if (!pLogo) {
-	    pLogo = XGetImage (dpy, mainWindow, 0, 0, 15, 16, AllPlanes, XYPixmap);
-	    FillImageByXPMData (pLogo, logo_xpm);
+	    rv = XpmCreateImageFromData (dpy, logo_xpm, &pLogo, &mask, &attrib);
+	    if (rv != XpmSuccess)
+		fprintf (stderr, "Failed to read xpm file: Logo\n");
 	}
 	if (!pVK) {
-	    pVK = XGetImage (dpy, mainWindow, 0, 0, 19, 16, AllPlanes, XYPixmap);
-	    FillImageByXPMData (pVK, vk_1_xpm);
+	    rv = XpmCreateImageFromData (dpy, vk_1_xpm, &pVK, &mask, &attrib);
+	    if (rv != XpmSuccess)
+		fprintf (stderr, "Failed to read xpm file: VK\n");
 	}
 
 	XPutImage (dpy, mainWindow, mainWindowColor.backGC, pLogo, 0, 0, 2, 2, 15, 16);
@@ -167,44 +187,50 @@ void DisplayMainWindow (void)
 	iPos = 20;
 	if (!bCompactMainWindow) {
 	    if (!pPunc[bChnPunc]) {
-		pPunc[bChnPunc] = XGetImage (dpy, mainWindow, 0, 0, 15, 16, AllPlanes, XYPixmap);
-		FillImageByXPMData (pPunc[bChnPunc], PuncLogo[bChnPunc]);
+		rv = XpmCreateImageFromData (dpy, PuncLogo[bChnPunc], &pPunc[bChnPunc], &mask, &attrib);
+		if (rv != XpmSuccess)
+		    fprintf (stderr, "Failed to read xpm file: Punc\n");
 	    }
 	    XPutImage (dpy, mainWindow, mainWindowColor.backGC, pPunc[bChnPunc], 0, 0, iPos, 2, 15, 16);
 	    iPos += 18;
 
 	    if (!pCorner[bCorner]) {
-		pCorner[bCorner] = XGetImage (dpy, mainWindow, 0, 0, 15, 16, AllPlanes, XYPixmap);
-		FillImageByXPMData (pCorner[bCorner], CornerLogo[bCorner]);
+		rv = XpmCreateImageFromData (dpy, CornerLogo[bCorner], &pCorner[bCorner], &mask, &attrib);
+		if (rv != XpmSuccess)
+		    fprintf (stderr, "Failed to read xpm file: Corner\n");
 	    }
 	    XPutImage (dpy, mainWindow, mainWindowColor.backGC, pCorner[bCorner], 0, 0, iPos, 2, 15, 16);
 	    iPos += 18;
 
 	    if (!pGBK[bUseGBK]) {
-		pGBK[bUseGBK] = XGetImage (dpy, mainWindow, 0, 0, 15, 16, AllPlanes, XYPixmap);
-		FillImageByXPMData (pGBK[bUseGBK], GBKLogo[bUseGBK]);
+		rv = XpmCreateImageFromData (dpy, GBKLogo[bUseGBK], &pGBK[bUseGBK], &mask, &attrib);
+		if (rv != XpmSuccess)
+		    fprintf (stderr, "Failed to read xpm file: GBKLogo\n");
 	    }
 	    XPutImage (dpy, mainWindow, mainWindowColor.backGC, pGBK[bUseGBK], 0, 0, iPos, 2, 15, 16);
 	    iPos += 18;
 
 	    if (!pLX[bUseLegend]) {
-		pLX[bUseLegend] = XGetImage (dpy, mainWindow, 0, 0, 15, 16, AllPlanes, XYPixmap);
-		FillImageByXPMData (pLX[bUseLegend], LXLogo[bUseLegend]);
+		rv = XpmCreateImageFromData (dpy, LXLogo[bUseLegend], &pLX[bUseLegend], &mask, &attrib);
+		if (rv != XpmSuccess)
+		    fprintf (stderr, "Failed to read xpm file: LXLogo\n");
 	    }
 	    XPutImage (dpy, mainWindow, mainWindowColor.backGC, pLX[bUseLegend], 0, 0, iPos, 2, 15, 16);
 	    iPos += 18;
 
 	    if (!pGBKT[bUseGBKT]) {
-		pGBKT[bUseGBKT] = XGetImage (dpy, mainWindow, 0, 0, 15, 16, AllPlanes, XYPixmap);
-		FillImageByXPMData (pGBKT[bUseGBKT], GBKTLogo[bUseGBKT]);
+		rv = XpmCreateImageFromData (dpy, GBKTLogo[bUseGBKT], &pGBKT[bUseGBKT], &mask, &attrib);
+		if (rv != XpmSuccess)
+		    fprintf (stderr, "Failed to read xpm file: GBKTLogo\n");
 	    }
 	    XPutImage (dpy, mainWindow, mainWindowColor.backGC, pGBKT[bUseGBKT], 0, 0, iPos, 2, 15, 16);
 	    iPos += 18;
 	}
 
 	if (!pLock[bLocked]) {
-	    pLock[bLocked] = XGetImage (dpy, mainWindow, 0, 0, 8, 16, AllPlanes, XYPixmap);
-	    FillImageByXPMData (pLock[bLocked], LockLogo[bLocked]);
+	    rv = XpmCreateImageFromData (dpy, LockLogo[bLocked], &pLock[bLocked], &mask, &attrib);
+	    if (rv != XpmSuccess)
+		fprintf (stderr, "Failed to read xpm file: LockLogo\n");
 	}
 	XPutImage (dpy, mainWindow, mainWindowColor.backGC, pLock[bLocked], 0, 0, iPos, 2, 15, 16);
 	iPos += 11;
@@ -220,29 +246,41 @@ void DisplayMainWindow (void)
 	iPos += 2;
 	p1 = (bVK) ? vks[iCurrentVK].strName : im[iIMIndex].strName;
 	while (*p1) {
-	    if (isprint (*p1))
-		bEn = True;
-	    else
-		bEn = False;
 	    p2 = strTemp;
-	    while (*p1) {
+	    if (isprint (*p1))	//ʹ
+		bEn = True;
+	    else {
 		*p2++ = *p1++;
+		*p2++ = *p1++;
+		bEn = False;
+	    }
+	    while (*p1) {
 		if (isprint (*p1)) {
 		    if (!bEn)
 			break;
+		    *p2++ = *p1++;
 		}
 		else {
 		    if (bEn)
 			break;
+		    *p2++ = *p1++;
+		    *p2++ = *p1++;
 		}
 	    }
 	    *p2 = '\0';
 
-	    OutputString (mainWindow, (bEn) ? xftMainWindowFontEn : xftMainWindowFont, strTemp, iPos, FontHeight (xftMainWindowFont) + (MAINWND_HEIGHT - FontHeight (xftMainWindowFont)) / 2 - 1, IMNameColor[iIndex].color);
-	    iPos += StringWidth (strTemp, (bEn) ? xftMainWindowFontEn : xftMainWindowFont);
+	    strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition (strTemp) : strTemp;
+	    OutputString (mainWindow, (bEn) ? xftMainWindowFontEn : xftMainWindowFont, strGBKT, iPos, FontHeight (xftMainWindowFont) + (MAINWND_HEIGHT - FontHeight (xftMainWindowFont)) / 2 - 1, IMNameColor[iIndex].color);
+	    iPos += StringWidth (strGBKT, (bEn) ? xftMainWindowFontEn : xftMainWindowFont);
+
+	    if (bUseGBKT)
+		free (strGBKT);
 	}
 #else
-	OutputString (mainWindow, fontSetMainWindow, (bVK) ? vks[iCurrentVK].strName : im[iIMIndex].strName, iPos, FontHeight (fontSetMainWindow) + (MAINWND_HEIGHT - FontHeight (fontSetMainWindow)) / 2 - 1, IMNameColor[iIndex].gc);
+	strGBKT = bUseGBKT ? ConvertGBKSimple2Tradition ((bVK) ? vks[iCurrentVK].strName : im[iIMIndex].strName) : ((bVK) ? vks[iCurrentVK].strName : im[iIMIndex].strName);
+	OutputString (mainWindow, fontSetMainWindow, strGBKT, iPos, FontHeight (fontSetMainWindow) + (MAINWND_HEIGHT - FontHeight (fontSetMainWindow)) / 2 - 1, IMNameColor[iIndex].gc);
+	if (bUseGBKT)
+	    free (strGBKT);
 #endif
 
 	if (_3DEffectMainWindow) {
@@ -302,11 +340,3 @@ void InitMainWindowColor (void)
 	XSetForeground (dpy, IMNameColor[i].gc, iPixel);
     }
 }
-
-void ChangeLock (void)
-{
-    bLocked = !bLocked;
-    DisplayMainWindow ();
-
-    SaveProfile ();
-}
