@@ -28,14 +28,9 @@
 #define MAX_USER_INPUT	300
 
 #define HOT_KEY_COUNT	2
-#define MAX_HZ_SAVED    1024
 
 #define MAX_IM_NAME	12
 
-#define NAME_OF_PINYIN		"ƴ"
-#define NAME_OF_SHUANGPIN	"˫ƴ"
-#define NAME_OF_QUWEI		"λ"
-
 #define TEMP_FILE		"FCITX_DICT_TEMP"
 
 typedef enum _SEARCH_MODE {
@@ -87,7 +82,9 @@ typedef enum _KEY_RELEASED {
     KR_OTHER = 0,
     KR_CTRL,
     KR_2ND_SELECTKEY,
-    KR_3RD_SELECTKEY
+    KR_2ND_SELECTKEY_OTHER,
+    KR_3RD_SELECTKEY,
+    KR_3RD_SELECTKEY_OTHER
 } KEY_RELEASED;
 
 typedef struct {
@@ -107,7 +104,7 @@ typedef int     HOTKEYS;
 void            ProcessKey (IMForwardEventStruct * call_data);
 void            ResetInput (void);
 void            CloseIM (IMForwardEventStruct * call_data);
-void            ChangeIMState ( CARD16 call_data);
+void            ChangeIMState (CARD16 call_data);
 Bool            IsHotKey (int iKey, HOTKEYS * hotkey);
 INPUT_RETURN_VALUE ChangeCorner (void);
 INPUT_RETURN_VALUE ChangePunc (void);
