@@ -56,8 +56,7 @@ INT8            iTableCount;
 Bool            bTableDictLoaded = False;	//Loads tables only if needed
 
 RECORD         *currentRecord = NULL, *recordHead = NULL;
-RECORD        **tableSingleHZ;	//Records the single characters in table to speed auto phrase
-unsigned int    iSingleHZCount;
+RECORD         *tableSingleHZ[SINGLE_HZ_COUNT];	//Records the single characters in table to speed auto phrase
 TABLECANDWORD   tableCandWord[MAX_CAND_WORD];
 
 RECORD_INDEX   *recordIndex = NULL;
@@ -65,8 +64,7 @@ RECORD_INDEX   *recordIndex = NULL;
 AUTOPHRASE     *autoPhrase = NULL;
 AUTOPHRASE     *insertPoint = NULL;
 
-INT16           iAutoPhrase = 0;
-INT16           iTotalAutoPhrase;
+uint            iAutoPhrase = 0;
 uint            iTableCandDisplayed;
 uint            iTableTotalCandCount;
 INT16           iTableOrderChanged = 0;
@@ -89,7 +87,7 @@ INT8            iTableNewPhraseHZCount;
 Bool            bCanntFindCode;	//Records if new phrase has corresponding code - should be always false
 char           *strNewPhraseCode;
 
-SINGLE_HZ       hzLastInput[MAX_HZ_SAVED + PHRASE_MAX_LENGTH];	//Records last HZ input
+SINGLE_HZ       hzLastInput[PHRASE_MAX_LENGTH];	//Records last HZ input
 INT16           iHZLastInputCount = 0;
 Bool            bTablePhraseTips = False;
 
@@ -138,14 +136,15 @@ extern uint     iFixedInputWindowWidth;
 //----------------------------------------
 extern PYFA    *PYFAList;
 extern PYCandWord PYCandWords[];
-extern Bool     bSingleHZMode;
+
+//extern Bool     bSingleHZMode;
 extern char     strFindString[];
 extern ParsePYStruct findMap;
 
 //----------------------------------------
 
 /*
- * ȡ뷨   ļ·
+ * ȡ뷨ƺļ·
  */
 void LoadTableInfo (void)
 {
@@ -201,6 +200,7 @@ void LoadTableInfo (void)
 	table[iTableIMIndex].bUsePY = True;
 	table[iTableIMIndex].cPinyin = '\0';
 	table[iTableIMIndex].bTableAutoSendToClient = True;
+	table[iTableIMIndex].rule = NULL;
 	table[iTableIMIndex].bUseMatchingKey = False;
 	table[iTableIMIndex].cMatchingKey = '\0';
 	table[iTableIMIndex].bTableExactMatch = False;
@@ -313,8 +313,6 @@ void LoadTableInfo (void)
 	}
     }
 
-    table[iTableIMIndex].rule = NULL;
-
     fclose (fp);
 }
 
@@ -326,9 +324,12 @@ Bool LoadTableDict (void)
     RECORD         *recTemp;
     char            strPath[PATH_MAX];
     unsigned int    i = 0;
-    unsigned int    iTemp;
+    unsigned int    iTemp, iTempCount;
     char            cChar = 0;
 
+#ifdef _DEBUG
+    fprintf (stderr, "LOAD Table Dict\n");
+#endif
     //ȣǸõĸ
     for (i = 0; i < iTableCount; i++) {
 	if (table[i].iIMIndex == iIMIndex)
@@ -391,17 +392,14 @@ Bool LoadTableDict (void)
     currentRecord = recordHead;
 
     fread (&(table[iTableIMIndex].iRecordCount), sizeof (unsigned int), 1, fpDict);
-    iSingleHZCount = 0;
+
+    for (i = 0; i < SINGLE_HZ_COUNT; i++)
+	tableSingleHZ[i] = (RECORD *) NULL;
 
     for (i = 0; i < table[iTableIMIndex].iRecordCount; i++) {
 	fread (strCode, sizeof (char), table[iTableIMIndex].iCodeLength + 1, fpDict);
 	fread (&iTemp, sizeof (unsigned int), 1, fpDict);
 	fread (strHZ, sizeof (char), iTemp, fpDict);
-
-	//iTempΪ3˵Ǹ
-	if (iTemp == 3)
-	    iSingleHZCount++;
-
 	recTemp = (RECORD *) malloc (sizeof (RECORD));
 	recTemp->strCode = (char *) malloc (sizeof (char) * (table[iTableIMIndex].iCodeLength + 1));
 	strcpy (recTemp->strCode, strCode);
@@ -413,7 +411,6 @@ Bool LoadTableDict (void)
 	fread (&(recTemp->iIndex), sizeof (unsigned int), 1, fpDict);
 	if (recTemp->iIndex > iTableIndex)
 	    iTableIndex = recTemp->iIndex;
-
 	/*  */
 	if (cChar != recTemp->strCode[0]) {
 	    cChar = recTemp->strCode[0];
@@ -423,7 +420,18 @@ Bool LoadTableDict (void)
 	    recordIndex[iTemp].record = recTemp;
 	}
 	/* **************************************************************** */
-
+	/** Ϊһ   */
+	if (strlen (recTemp->strHZ) == 2 && !IsIgnoreChar (strCode[0])) {
+	    iTemp = CalHZIndex (recTemp->strHZ);
+	    if (iTemp >= 0 && iTemp < SINGLE_HZ_COUNT) {
+		if (tableSingleHZ[iTemp]) {
+		    if (strlen (strCode) > strlen (tableSingleHZ[iTemp]->strCode))
+			tableSingleHZ[iTemp] = recTemp;
+		}
+		else
+		    tableSingleHZ[iTemp] = recTemp;
+	    }
+	}
 	currentRecord->next = recTemp;
 	recTemp->prev = currentRecord;
 	currentRecord = recTemp;
@@ -433,7 +441,6 @@ Bool LoadTableDict (void)
     recordHead->prev = currentRecord;
 
     fclose (fpDict);
-
     //ȡӦű
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/");
@@ -463,37 +470,48 @@ Bool LoadTableDict (void)
     strNewPhraseCode[table[iTableIMIndex].iCodeLength] = '\0';
     bTableDictLoaded = True;
 
-    /*
-     * Ҫٸ¼ԶɵĴ
-     */
-    iTotalAutoPhrase = 0;
-    for (i = 2; i <= table[iTableIMIndex].iAutoPhrase; i++)
-	iTotalAutoPhrase += MAX_HZ_SAVED - i + 1;
+    //ΪԶռ
+    autoPhrase = (AUTOPHRASE *) malloc (sizeof (AUTOPHRASE) * AUTO_PHRASE_COUNT);
 
-    autoPhrase = (AUTOPHRASE *) malloc (sizeof (AUTOPHRASE) * iTotalAutoPhrase);
+    //ȡϴαԶϢ
+    strcpy (strPath, (char *) getenv ("HOME"));
+    strcat (strPath, "/.fcitx/");
+    strcat (strPath, table[iTableIMIndex].strName);
+    strcat (strPath, "_LastAutoPhrase.tmp");
+    fpDict = fopen (strPath, "rb");
+    i = 0;
+    if (fpDict) {
+	fread (&iAutoPhrase, sizeof (unsigned int), 1, fpDict);
 
-    for (i = 0; i < iTotalAutoPhrase; i++) {
+	for (; i < iAutoPhrase; i++) {
+	    autoPhrase[i].strCode = (char *) malloc (sizeof (char) * (table[iTableIMIndex].iCodeLength + 1));
+	    autoPhrase[i].strHZ = (char *) malloc (sizeof (char) * (PHRASE_MAX_LENGTH * 2 + 1));
+	    fread (autoPhrase[i].strCode, table[iTableIMIndex].iCodeLength + 1, 1, fpDict);
+	    fread (autoPhrase[i].strHZ, PHRASE_MAX_LENGTH * 2 + 1, 1, fpDict);
+	    fread (&iTempCount, sizeof (unsigned int), 1, fpDict);
+	    autoPhrase[i].iSelected = iTempCount;
+	    if (i == AUTO_PHRASE_COUNT - 1)
+		autoPhrase[i].next = &autoPhrase[0];
+	    else
+		autoPhrase[i].next = &autoPhrase[i + 1];
+	}
+	fclose (fpDict);
+    }
+
+    for (; i < AUTO_PHRASE_COUNT; i++) {
 	autoPhrase[i].strCode = (char *) malloc (sizeof (char) * (table[iTableIMIndex].iCodeLength + 1));
 	autoPhrase[i].strHZ = (char *) malloc (sizeof (char) * (PHRASE_MAX_LENGTH * 2 + 1));
 	autoPhrase[i].iSelected = 0;
-	if (i == iTotalAutoPhrase - 1)
+	if (i == AUTO_PHRASE_COUNT - 1)
 	    autoPhrase[i].next = &autoPhrase[0];
 	else
 	    autoPhrase[i].next = &autoPhrase[i + 1];
     }
-    insertPoint = &autoPhrase[0];
 
-    /*
-     * Ϊһ
-     */
-    tableSingleHZ = (RECORD **) malloc (sizeof (RECORD *) * iSingleHZCount);
-    recTemp = recordHead->next;
-    i = 0;
-    while (recTemp != recordHead) {
-	if (strlen (recTemp->strHZ) == 2)
-	    tableSingleHZ[i++] = recTemp;
-	recTemp = recTemp->next;
-    }
+    if (i == AUTO_PHRASE_COUNT)
+	insertPoint = &autoPhrase[0];
+    else
+	insertPoint = &autoPhrase[i - 1];
 
     /*
      * Ȼʼ뷨
@@ -523,6 +541,9 @@ Bool LoadTableDict (void)
        fclose(temp2);
        fclose(temp1);    */
     /* ********************************************** */
+#ifdef _DEBUG
+    fprintf (stderr, "LOAD Table Dict OK!\n");
+#endif
 
     return True;
 }
@@ -531,7 +552,6 @@ void TableInit (void)
 {
     bSP = False;
     PYBaseOrder = baseOrder;
-    baseOrder = AD_FAST;
     strPYAuto[0] = '\0';
 }
 
@@ -592,16 +612,13 @@ void FreeTableIM (void)
     }
 
     //ͷԶĿռ
-    for (i = 0; i < iTotalAutoPhrase; i++) {
+    for (i = 0; i < AUTO_PHRASE_COUNT; i++) {
 	free (autoPhrase[i].strCode);
 	free (autoPhrase[i].strHZ);
     }
     free (autoPhrase);
 
     baseOrder = PYBaseOrder;
-
-    //ͷŵֱ
-    free (tableSingleHZ);
 }
 
 void TableResetStatus (void)
@@ -610,7 +627,7 @@ void TableResetStatus (void)
     bIsTableDelPhrase = False;
     bIsTableAdjustOrder = False;
     bIsDoInputOnly = False;
-    bSingleHZMode = True;
+    //bSingleHZMode = False;
 }
 
 void SaveTableDict (void)
@@ -678,6 +695,30 @@ void SaveTableDict (void)
 
     iTableOrderChanged = 0;
     iTableChanged = 0;
+
+    //ϴεԶϢ
+    strcpy (strPathTemp, (char *) getenv ("HOME"));
+    strcat (strPathTemp, "/.fcitx/");
+    strcat (strPathTemp, TEMP_FILE);
+    fpDict = fopen (strPathTemp, "wb");
+    if (fpDict) {
+	fwrite (&iAutoPhrase, sizeof (int), 1, fpDict);
+	for (i = 0; i < iAutoPhrase; i++) {
+	    fwrite (autoPhrase[i].strCode, table[iTableIMIndex].iCodeLength + 1, 1, fpDict);
+	    fwrite (autoPhrase[i].strHZ, PHRASE_MAX_LENGTH * 2 + 1, 1, fpDict);
+	    iTemp = autoPhrase[i].iSelected;
+	    fwrite (&iTemp, sizeof (unsigned int), 1, fpDict);
+	}
+	fclose (fpDict);
+    }
+
+    strcpy (strPath, (char *) getenv ("HOME"));
+    strcat (strPath, "/.fcitx/");
+    strcat (strPath, table[iTableIMIndex].strName);
+    strcat (strPath, "_LastAutoPhrase.tmp");
+    if (access (strPath, 0))
+	unlink (strPath);
+    rename (strPathTemp, strPath);
 }
 
 Bool IsInputKey (int iKey)
@@ -740,7 +781,7 @@ INPUT_RETURN_VALUE DoTableInput (int iKey)
 
 	if (!bIsTableAddPhrase && !bIsTableDelPhrase && !bIsTableAdjustOrder) {
 	    if (strCodeInput[0] == table[iTableIMIndex].cPinyin && table[iTableIMIndex].bUsePY) {
-		if (iCodeInputCount != (MAX_PY_LENGTH + 1)) {
+		if (iCodeInputCount != (MAX_PY_LENGTH * 5 + 1)) {
 		    strCodeInput[iCodeInputCount++] = (char) iKey;
 		    strCodeInput[iCodeInputCount] = '\0';
 		    retVal = TableGetCandWords (SM_FIRST);
@@ -764,7 +805,7 @@ INPUT_RETURN_VALUE DoTableInput (int iKey)
 			strTemp = GetPunc (strCodeInput[0]);
 
 			if (table[iTableIMIndex].bTableAutoSendToClient && (iCodeInputCount == table[iTableIMIndex].iCodeLength)) {
-			    if (iCandWordCount == 1 && (tableCandWord[0].flag || (!tableCandWord[0].flag && !table[iTableIMIndex].iSaveAutoPhraseAfter))) {	//ֻһѡʣ͵ͻ
+			    if (iCandWordCount == 1 && (tableCandWord[0].flag != CT_AUTOPHRASE || (tableCandWord[0].flag == CT_AUTOPHRASE && !table[iTableIMIndex].iSaveAutoPhraseAfter))) {	//ֻһѡʣ͵ͻ
 				strcpy (strStringGet, TableGetCandWord (0));
 				iCandWordCount = 0;
 				if (bIsInLegend)
@@ -782,7 +823,7 @@ INPUT_RETURN_VALUE DoTableInput (int iKey)
 		else {
 		    if (table[iTableIMIndex].bTableAutoSendToClient) {
 			if (iCandWordCount) {
-			    if (tableCandWord[0].flag) {
+			    if (tableCandWord[0].flag != CT_AUTOPHRASE) {
 				strcpy (strStringGet, TableGetCandWord (0));
 				retVal = IRV_GET_CANDWORDS_NEXT;
 			    }
@@ -958,7 +999,7 @@ INPUT_RETURN_VALUE DoTableInput (int iKey)
 		messageUp[0].type = MSG_TIPS;
 		retVal = IRV_DISPLAY_MESSAGE;
 	    }
-	    else if (iKey == (XK_BackSpace & 0x00FF) || iKey==CTRL_H) {
+	    else if (iKey == (XK_BackSpace & 0x00FF) || iKey == CTRL_H) {
 		if (!iCodeInputCount) {
 		    bIsInLegend = False;
 		    return IRV_DONOT_PROCESS_CLEAN;
@@ -1030,7 +1071,7 @@ INPUT_RETURN_VALUE DoTableInput (int iKey)
 
 char           *TableGetCandWord (int iIndex)
 {
-    char           *pCandWord;
+    char           *pCandWord = NULL;
 
     if (!strcmp (strCodeInput, table[iTableIMIndex].strSymbol))
 	return TableGetFHCandWord (iIndex);
@@ -1043,18 +1084,20 @@ char           *TableGetCandWord (int iIndex)
     if (iIndex > (iCandWordCount - 1))
 	iIndex = iCandWordCount - 1;
 
-    if (tableCandWord[iIndex].flag) {
+    if (tableCandWord[iIndex].flag == CT_NORMAL) {
 	tableCandWord[iIndex].candWord.record->iHit++;
 	tableCandWord[iIndex].candWord.record->iIndex = ++iTableIndex;
     }
-    if (table[iTableIMIndex].tableOrder != AD_NO) {
-	iTableOrderChanged++;
-	if (iTableOrderChanged == TABLE_AUTO_SAVE_AFTER)
-	    SaveTableDict ();
-    }
-    if (tableCandWord[iIndex].flag)
+
+    iTableOrderChanged++;
+    if (iTableOrderChanged == TABLE_AUTO_SAVE_AFTER)
+	SaveTableDict ();
+
+    switch (tableCandWord[iIndex].flag) {
+    case CT_NORMAL:
 	pCandWord = tableCandWord[iIndex].candWord.record->strHZ;
-    else {
+	break;
+    case CT_AUTOPHRASE:
 	if (table[iTableIMIndex].iSaveAutoPhraseAfter) {
 	    if (table[iTableIMIndex].iSaveAutoPhraseAfter >= tableCandWord[iIndex].candWord.autoPhrase->iSelected)
 		tableCandWord[iIndex].candWord.autoPhrase->iSelected++;
@@ -1062,6 +1105,12 @@ char           *TableGetCandWord (int iIndex)
 		TableInsertPhrase (tableCandWord[iIndex].candWord.autoPhrase->strCode, tableCandWord[iIndex].candWord.autoPhrase->strHZ);
 	}
 	pCandWord = tableCandWord[iIndex].candWord.autoPhrase->strHZ;
+	break;
+    case CT_PYPHRASE:
+	pCandWord = tableCandWord[iIndex].candWord.strPYPhrase;
+	break;
+    default:
+	;
     }
 
     if (bUseLegend) {
@@ -1078,7 +1127,7 @@ char           *TableGetCandWord (int iIndex)
 
 	    strcpy (messageDown[0].strMsg, pCandWord);
 	    messageDown[0].type = MSG_TIPS;
-	    temp = TableFindCode (pCandWord, False);
+	    temp = tableSingleHZ[CalHZIndex (pCandWord)];
 	    if (temp) {
 		strcpy (messageDown[1].strMsg, temp->strCode);
 		messageDown[1].type = MSG_CODE;
@@ -1108,7 +1157,6 @@ char           *TableGetCandWord (int iIndex)
 INPUT_RETURN_VALUE TableGetPinyinCandWords (SEARCH_MODE mode)
 {
     int             i;
-    RECORD         *pRec;
 
     if (mode == SM_FIRST) {
 	strcpy (strFindString, strCodeInput + 1);
@@ -1126,12 +1174,8 @@ INPUT_RETURN_VALUE TableGetPinyinCandWords (SEARCH_MODE mode)
 
     //潫ƴĺѡֱתΪ뷨ʽ
     for (i = 0; i < iCandWordCount; i++) {
-	pRec = TableFindCode (PYFAList[PYCandWords[i].cand.base.iPYFA].pyBase[PYCandWords[i].cand.base.iBase].strHZ, False);
-	tableCandWord[i].flag = True;
-	if (pRec)
-	    tableCandWord[i].candWord.record = pRec;
-	else
-	    tableCandWord[i].candWord.record = recordHead->next;
+	tableCandWord[i].flag = CT_PYPHRASE;
+	PYGetCandText (i, tableCandWord[i].candWord.strPYPhrase);
     }
 
     return IRV_DISPLAY_CANDWORDS;
@@ -1139,8 +1183,9 @@ INPUT_RETURN_VALUE TableGetPinyinCandWords (SEARCH_MODE mode)
 
 INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 {
-    int             i;
+    int             i, iTemp;
     char            strTemp[3], *pstr;
+    RECORD         *recTemp;
 
     if (bIsInLegend)
 	return TableGetLegendCandWords (mode);
@@ -1185,9 +1230,9 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 
 	iCandWordCount = 0;
 	if (mode == SM_PREV && table[iTableIMIndex].bRule && table[iTableIMIndex].bAutoPhrase && iCodeInputCount == table[iTableIMIndex].iCodeLength) {
-	    for (i = 0; i < iAutoPhrase; i++) {
+	    for (i = iAutoPhrase - 1; i >= 0; i--) {
 		if (!TableCompareCode (strCodeInput, autoPhrase[i].strCode) && autoPhrase[i].flag) {
-		    if (!TableCandHasPhrase (autoPhrase[i].strHZ))
+		    if (TableHasPhrase (autoPhrase[i].strCode, autoPhrase[i].strHZ))
 			TableAddAutoCandWord (i, mode);
 		}
 	    }
@@ -1208,9 +1253,9 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 	}
 
 	if (table[iTableIMIndex].bRule && table[iTableIMIndex].bAutoPhrase && mode != SM_PREV && iCodeInputCount == table[iTableIMIndex].iCodeLength) {
-	    for (i = 0; i < iAutoPhrase; i++) {
+	    for (i = iAutoPhrase - 1; i >= 0; i--) {
 		if (!TableCompareCode (strCodeInput, autoPhrase[i].strCode) && !autoPhrase[i].flag) {
-		    if (!TableCandHasPhrase (autoPhrase[i].strHZ)) {
+		    if (TableHasPhrase (autoPhrase[i].strCode, autoPhrase[i].strHZ)) {
 			if (mode == SM_FIRST)
 			    iTableTotalCandCount++;
 			TableAddAutoCandWord (i, mode);
@@ -1249,19 +1294,53 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 		if (i == 9)
 		    strTemp[0] = '0';
 
-		if (HasMatchingKey () || strCodeInput[0] == table[iTableIMIndex].cPinyin)
-		    pstr = (tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode;
+		if (HasMatchingKey ())
+		    pstr = (tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode;
+		/*else if (tableCandWord[i].flag==CT_PYPHRASE) {
+		   if (strlen(tableCandWord[i].candWord.strPYPhrase)==2){
+		   recTemp = tableSingleHZ[CalHZIndex (tableCandWord[i].candWord.strPYPhrase)];
+		   if (!recTemp)
+		   pstr=(char *)NULL;
+		   else
+		   pstr=recTemp->strCode;
+		   }
+		   else
+		   pstr=(char *)NULL;
+		   } */
 		else
-		    pstr = ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode) + iCodeInputCount;
+		    pstr = ((tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode) + iCodeInputCount;
+
 #ifdef _USE_XFT
-		iWidth += StringWidth (pstr, xftFontEn);
+		if (pstr)
+		    iWidth += StringWidth (pstr, xftFontEn);
 		iWidth += StringWidth (strTemp, xftFontEn);
-		iWidth += StringWidth ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ, xftFont);
+		switch (tableCandWord[i].flag) {
+		case CT_NORMAL:
+		    iWidth += StringWidth (tableCandWord[i].candWord.record->strHZ, xftFont);
+		    break;
+		case CT_AUTOPHRASE:
+		    iWidth += StringWidth (tableCandWord[i].candWord.autoPhrase->strHZ, xftFont);
+		    /*      case CT_PYPHRASE:
+		       iWidth += StringWidth (tableCandWord[i].candWord.strPhrase, xftFont); */
+		default:
+		    ;
+		}
 		iWidth += StringWidth (" ", xftFontEn);
 #else
-		iWidth += StringWidth (pstr, fontSet);
+		if (pstr)
+		    iWidth += StringWidth (pstr, fontSet);
 		iWidth += StringWidth (strTemp, fontSet);
-		iWidth += StringWidth ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ, fontSet);
+		switch (tableCandWord[i].flag) {
+		case CT_NORMAL:
+		    iWidth += StringWidth (tableCandWord[i].candWord.record->strHZ, fontSet);
+		    break;
+		case CT_AUTOPHRASE:
+		    iWidth += StringWidth (tableCandWord[i].candWord.autoPhrase->strHZ, fontSet);
+		    /*case CT_PYPHRASE:
+		       iWidth += StringWidth (tableCandWord[i].candWord.strPhrase, fontSet); */
+		default:
+		    ;
+		}
 		iWidth += StringWidth (" ", fontSet);
 #endif
 
@@ -1284,14 +1363,14 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 	    if (i < 0)
 		i = 0;
 	    for (iWidth = 0; iWidth < i; iWidth++) {
-		if (tableCandWord[iWidth].flag)
+		if (tableCandWord[iWidth].flag == CT_NORMAL)
 		    tableCandWord[iWidth].candWord.record->flag = True;
 		else
 		    tableCandWord[iWidth].candWord.autoPhrase->flag = True;
 	    }
 	    for (iWidth = 0; iWidth < (iCandWordCount - i); iWidth++) {
 		tableCandWord[iWidth].flag = tableCandWord[iWidth + i].flag;
-		if (tableCandWord[iWidth].flag)
+		if (tableCandWord[iWidth].flag == CT_NORMAL)
 		    tableCandWord[iWidth].candWord.record = tableCandWord[iWidth + i].candWord.record;
 		else
 		    tableCandWord[iWidth].candWord.autoPhrase = tableCandWord[iWidth + i].candWord.autoPhrase;
@@ -1301,25 +1380,26 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 	}
 	else {
 	    //󷭣Ҫǰ㳤
+	    iTemp = iCandWordCount;
 	    for (i = 0; i < iCandWordCount; i++) {
 		strTemp[0] = i + 1 + '0';
 		if (i == 9)
 		    strTemp[0] = '0';
 
-		if (HasMatchingKey () || strCodeInput[0] == table[iTableIMIndex].cPinyin)
-		    pstr = (tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode;
+		if (HasMatchingKey ())
+		    pstr = (tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode;
 		else
-		    pstr = ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode) + iCodeInputCount;
+		    pstr = ((tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode) + iCodeInputCount;
 
 #ifdef _USE_XFT
 		iWidth += StringWidth (pstr, xftFontEn);
 		iWidth += StringWidth (strTemp, xftFontEn);
-		iWidth += StringWidth ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ, xftFont);
+		iWidth += StringWidth ((tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ, xftFont);
 		iWidth += StringWidth (" ", xftFontEn);
 #else
 		iWidth += StringWidth (pstr, fontSet);
 		iWidth += StringWidth (strTemp, fontSet);
-		iWidth += StringWidth ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ, fontSet);
+		iWidth += StringWidth ((tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ, fontSet);
 		iWidth += StringWidth (" ", fontSet);
 #endif
 		if (iWidth > iFixedInputWindowWidth) {
@@ -1337,13 +1417,14 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 	    }
 
 	    for (iWidth = i; iWidth < iCandWordCount; iWidth++) {
-		if (tableCandWord[iWidth].flag)
+		if (tableCandWord[iWidth].flag == CT_NORMAL)
 		    tableCandWord[iWidth].candWord.record->flag = False;
 		else
 		    tableCandWord[iWidth].candWord.autoPhrase->flag = False;
 	    }
 
 	    iCandWordCount = i;
+	    iTableCandDisplayed -= iTemp - i;
 	}
     }
 
@@ -1355,16 +1436,46 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
 	strcpy (messageDown[uMessageDown].strMsg, strTemp);
 	messageDown[uMessageDown++].type = MSG_INDEX;
 
-	strcpy (messageDown[uMessageDown].strMsg, (tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strHZ : tableCandWord[i].candWord.autoPhrase->strHZ);
-	if (tableCandWord[i].flag)
-	    messageDown[uMessageDown++].type = ((i == 0) ? MSG_FIRSTCAND : MSG_OTHER);
-	else
+	switch (tableCandWord[i].flag) {
+	case CT_NORMAL:
+	    strcpy (messageDown[uMessageDown].strMsg, tableCandWord[i].candWord.record->strHZ);
+	    break;
+	case CT_AUTOPHRASE:
+	    strcpy (messageDown[uMessageDown].strMsg, tableCandWord[i].candWord.autoPhrase->strHZ);
+	    break;
+	case CT_PYPHRASE:
+	    strcpy (messageDown[uMessageDown].strMsg, tableCandWord[i].candWord.strPYPhrase);
+	    break;
+	default:
+	    ;
+	}
+
+	if (tableCandWord[i].flag == CT_AUTOPHRASE)
 	    messageDown[uMessageDown++].type = MSG_TIPS;
-	if (HasMatchingKey () || strCodeInput[0] == table[iTableIMIndex].cPinyin)
-	    pstr = (tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode;
 	else
-	    pstr = ((tableCandWord[i].flag) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode) + iCodeInputCount;
-	strcpy (messageDown[uMessageDown].strMsg, pstr);
+	    messageDown[uMessageDown++].type = ((i == 0) ? MSG_FIRSTCAND : MSG_OTHER);
+
+	if (tableCandWord[i].flag == CT_PYPHRASE) {
+	    if (strlen (tableCandWord[i].candWord.strPYPhrase) == 2) {
+		recTemp = tableSingleHZ[CalHZIndex (tableCandWord[i].candWord.strPYPhrase)];
+
+		if (!recTemp)
+		    pstr = (char *) NULL;
+		else
+		    pstr = recTemp->strCode;
+	    }
+	    else
+		pstr = (char *) NULL;
+	}
+	else if (HasMatchingKey ())
+	    pstr = (tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode;
+	else
+	    pstr = ((tableCandWord[i].flag == CT_NORMAL) ? tableCandWord[i].candWord.record->strCode : tableCandWord[i].candWord.autoPhrase->strCode) + iCodeInputCount;
+
+	if (pstr)
+	    strcpy (messageDown[uMessageDown].strMsg, pstr);
+	else
+	    messageDown[uMessageDown].strMsg[0] = '\0';
 
 	if (i != (iCandWordCount - 1)) {
 	    strcat (messageDown[uMessageDown].strMsg, " ");
@@ -1375,24 +1486,6 @@ INPUT_RETURN_VALUE TableGetCandWords (SEARCH_MODE mode)
     return IRV_DISPLAY_CANDWORDS;
 }
 
-/*
- * жϺѡбǷָĴ
- * ڽԶеظȥ
- */
-Bool TableCandHasPhrase (char *strHZ)
-{
-    int             i;
-
-    for (i = 0; i < iCandWordCount; i++) {
-	if (!tableCandWord[i].flag)
-	    return False;
-	if (!strcmp (strHZ, tableCandWord[i].candWord.record->strHZ))
-	    return True;
-    }
-
-    return False;
-}
-
 void TableAddAutoCandWord (INT16 which, SEARCH_MODE mode)
 {
     int             i, j;
@@ -1405,14 +1498,14 @@ void TableAddAutoCandWord (INT16 which, SEARCH_MODE mode)
 	}
 	else
 	    i = iCandWordCount++;
-	tableCandWord[i].flag = False;
+	tableCandWord[i].flag = CT_AUTOPHRASE;
 	tableCandWord[i].candWord.autoPhrase = &autoPhrase[which];
     }
     else {
 	if (iCandWordCount == iMaxCandWord)
 	    return;
 
-	tableCandWord[iCandWordCount].flag = False;
+	tableCandWord[iCandWordCount].flag = CT_AUTOPHRASE;
 	tableCandWord[iCandWordCount++].candWord.autoPhrase = &autoPhrase[which];
     }
 }
@@ -1428,7 +1521,7 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
 		i = iCandWordCount - 1;
 	    else {
 		for (i = 0; i < iCandWordCount; i++) {
-		    if (!tableCandWord[i].flag)
+		    if (tableCandWord[i].flag != CT_NORMAL)
 			break;
 		}
 	    }
@@ -1436,7 +1529,7 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
 	else {
 	    if (iCandWordCount == iMaxCandWord)
 		return;
-	    tableCandWord[iCandWordCount].flag = True;
+	    tableCandWord[iCandWordCount].flag = CT_NORMAL;
 	    tableCandWord[iCandWordCount++].candWord.record = record;
 	    return;
 	}
@@ -1444,7 +1537,7 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
     case AD_FREQ:
 	if (mode == SM_PREV) {
 	    for (i = iCandWordCount - 1; i >= 0; i--) {
-		if (tableCandWord[i].flag) {
+		if (tableCandWord[i].flag == CT_NORMAL) {
 		    if (strcmp (tableCandWord[i].candWord.record->strCode, record->strCode) < 0 || (strcmp (tableCandWord[i].candWord.record->strCode, record->strCode) == 0 && tableCandWord[i].candWord.record->iHit >= record->iHit))
 			break;
 		}
@@ -1470,7 +1563,7 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
     case AD_FAST:
 	if (mode == SM_PREV) {
 	    for (i = iCandWordCount - 1; i >= 0; i--) {
-		if (tableCandWord[i].flag) {
+		if (tableCandWord[i].flag == CT_NORMAL) {
 		    if (strcmp (tableCandWord[i].candWord.record->strCode, record->strCode) < 0 || (strcmp (tableCandWord[i].candWord.record->strCode, record->strCode) == 0 && tableCandWord[i].candWord.record->iIndex >= record->iIndex))
 			break;
 		}
@@ -1499,7 +1592,7 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
 	if (iCandWordCount == iMaxCandWord) {
 	    for (j = 0; j < i; j++) {
 		tableCandWord[j].flag = tableCandWord[j + 1].flag;
-		if (tableCandWord[j].flag)
+		if (tableCandWord[j].flag == CT_NORMAL)
 		    tableCandWord[j].candWord.record = tableCandWord[j + 1].candWord.record;
 		else
 		    tableCandWord[j].candWord.autoPhrase = tableCandWord[j + 1].candWord.autoPhrase;
@@ -1508,7 +1601,7 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
 	else {
 	    for (j = iCandWordCount; j > i; j--) {
 		tableCandWord[j].flag = tableCandWord[j - 1].flag;
-		if (tableCandWord[j].flag)
+		if (tableCandWord[j].flag == CT_NORMAL)
 		    tableCandWord[j].candWord.record = tableCandWord[j - 1].candWord.record;
 		else
 		    tableCandWord[j].candWord.autoPhrase = tableCandWord[j - 1].candWord.autoPhrase;
@@ -1521,14 +1614,14 @@ void TableAddCandWord (RECORD * record, SEARCH_MODE mode)
 	    j--;
 	for (; j > i; j--) {
 	    tableCandWord[j].flag = tableCandWord[j - 1].flag;
-	    if (tableCandWord[j].flag)
+	    if (tableCandWord[j].flag == CT_NORMAL)
 		tableCandWord[j].candWord.record = tableCandWord[j - 1].candWord.record;
 	    else
 		tableCandWord[j].candWord.autoPhrase = tableCandWord[j - 1].candWord.autoPhrase;
 	}
     }
 
-    tableCandWord[i].flag = True;
+    tableCandWord[i].flag = CT_NORMAL;
     tableCandWord[i].candWord.record = record;
 
     if (iCandWordCount != iMaxCandWord)
@@ -1554,7 +1647,7 @@ void TableSetCandWordsFlag (int iCount, Bool flag)
     int             i;
 
     for (i = 0; i < iCount; i++) {
-	if (tableCandWord[i].flag)
+	if (tableCandWord[i].flag == CT_NORMAL)
 	    tableCandWord[i].candWord.record->flag = flag;
 	else
 	    tableCandWord[i].candWord.autoPhrase->flag = flag;
@@ -1628,7 +1721,7 @@ int TableFindFirstMatchCode (void)
  * 
  * bMode=Trueʾʣʱһ롣ֻжʱض룬һ
  */
-RECORD         *TableFindCode (char *strHZ, Bool bMode)
+/*RECORD         *TableFindCode (char *strHZ, Bool bMode)
 {
     RECORD         *recShort = NULL;	//¼λ
     int             i;
@@ -1646,7 +1739,7 @@ RECORD         *TableFindCode (char *strHZ, Bool bMode)
     }
 
     return recShort;
-}
+}*/
 
 /*
  * ŵ˳Ŵ1ʼ
@@ -1655,9 +1748,9 @@ RECORD         *TableFindCode (char *strHZ, Bool bMode)
 void TableAdjustOrderByIndex (int iIndex)
 {
     RECORD         *recTemp;
-    int		iTemp;
+    int             iTemp;
 
-    if (!(tableCandWord[iIndex - 1].flag))
+    if (tableCandWord[iIndex - 1].flag != CT_NORMAL)
 	return;
 
     recTemp = tableCandWord[iIndex - 1].candWord.record;
@@ -1675,17 +1768,17 @@ void TableAdjustOrderByIndex (int iIndex)
     tableCandWord[iIndex - 1].candWord.record->next = recTemp;
 
     iTableChanged++;
-    
+
     //ҪĻ
-    if ( tableCandWord[iIndex - 1].candWord.record->strCode[1]=='\0') {
-        for (iTemp = 0; iTemp < strlen (table[iTableIMIndex].strInputCode); iTemp++) {
-		if ( recordIndex[iTemp].cCode == tableCandWord[iIndex - 1].candWord.record->strCode[0] ) {
-			recordIndex[iTemp].record = tableCandWord[iIndex - 1].candWord.record;
-			break;
-		}
+    if (tableCandWord[iIndex - 1].candWord.record->strCode[1] == '\0') {
+	for (iTemp = 0; iTemp < strlen (table[iTableIMIndex].strInputCode); iTemp++) {
+	    if (recordIndex[iTemp].cCode == tableCandWord[iIndex - 1].candWord.record->strCode[0]) {
+		recordIndex[iTemp].record = tableCandWord[iIndex - 1].candWord.record;
+		break;
+	    }
 	}
     }
-    
+
     if (iTableChanged == 5)
 	SaveTableDict ();
 }
@@ -1695,7 +1788,7 @@ void TableAdjustOrderByIndex (int iIndex)
  */
 void TableDelPhraseByIndex (int iIndex)
 {
-    if (!(tableCandWord[iIndex - 1].flag))
+    if (tableCandWord[iIndex - 1].flag != CT_NORMAL)
 	return;
 
     if (strlen (tableCandWord[iIndex - 1].candWord.record->strHZ) <= 2)
@@ -1771,7 +1864,7 @@ RECORD         *TableFindPhrase (char *strHZ)
     strTemp[1] = strHZ[1];
     strTemp[2] = '\0';
 
-    recTemp = TableFindCode (strTemp, True);
+    recTemp = tableSingleHZ[CalHZIndex (strTemp)];
     if (!recTemp)
 	return (RECORD *) NULL;
 
@@ -1871,7 +1964,7 @@ void TableCreatePhraseCode (char *strHZ)
 	    strTemp[1] = strHZ[(iLen - table[iTableIMIndex].rule[i].rule[i1].iWhich) * 2 + 1];
 	}
 
-	recTemp = TableFindCode (strTemp, True);
+	recTemp = tableSingleHZ[CalHZIndex (strTemp)];
 
 	if (!recTemp) {
 	    bCanntFindCode = True;
@@ -2020,7 +2113,7 @@ void TableAddLegendCandWord (RECORD * record, SEARCH_MODE mode)
 	    tableCandWord[j].candWord.record = tableCandWord[j - 1].candWord.record;
     }
 
-    tableCandWord[i].flag = True;
+    tableCandWord[i].flag = CT_NORMAL;
     tableCandWord[i].candWord.record = record;
 
     if (iLegendCandWordCount != iMaxCandWord)
@@ -2191,7 +2284,7 @@ void TableCreateAutoPhrase (INT8 iCount)
 	    }
 
 	    TableCreatePhraseCode (strHZ);
-	    if (iAutoPhrase != iTotalAutoPhrase) {
+	    if (iAutoPhrase != AUTO_PHRASE_COUNT) {
 		autoPhrase[iAutoPhrase].flag = False;
 		strcpy (autoPhrase[iAutoPhrase].strCode, strNewPhraseCode);
 		strcpy (autoPhrase[iAutoPhrase].strHZ, strHZ);
@@ -2211,11 +2304,11 @@ void TableCreateAutoPhrase (INT8 iCount)
     }
 
     /*
-    for (i=0;i<iAutoPhrase;i++ ) {
+       for (i=0;i<iAutoPhrase;i++ ) {
        printf("%d: %s  %s\n",i+1,autoPhrase[i].strCode,autoPhrase[i].strHZ);
        puts("===========================================");
-    }
-    */
+       }
+     */
 }
 
 void UpdateHZLastInput (char *str)
@@ -2223,7 +2316,7 @@ void UpdateHZLastInput (char *str)
     int             i, j;
 
     for (i = 0; i < strlen (str) / 2; i++) {
-	if (iHZLastInputCount < MAX_HZ_SAVED)
+	if (iHZLastInputCount < PHRASE_MAX_LENGTH)
 	    iHZLastInputCount++;
 	else {
 	    for (j = 0; j < (iHZLastInputCount - 1); j++) {
