@@ -31,6 +31,8 @@
 #define PHRASE_MAX_LENGTH 10
 #define FH_MAX_LENGTH  10
 #define TABLE_AUTO_SAVE_AFTER 1024
+#define AUTO_PHRASE_COUNT 10000
+#define SINGLE_HZ_COUNT 33000
 
 typedef struct _RULE_RULE {
     unsigned char   iFlag;	// 1 -->    0 --> 
@@ -61,15 +63,15 @@ typedef struct _TABLE {
     unsigned int    iRecordCount;
     ADJUSTORDER     tableOrder;
 
-    Bool            bUsePY;			//ʹƴ
+    Bool            bUsePY;	//ʹƴ
     Bool            bTableAutoSendToClient;	//Զ
-    Bool            bUseMatchingKey;		//Ƿģƥ
-    Bool            bAutoPhrase;		//ǷԶ
+    Bool            bUseMatchingKey;	//Ƿģƥ
+    Bool            bAutoPhrase;	//ǷԶ
     INT8            iSaveAutoPhraseAfter;	//ѡNκ󱣴Զ飬0-棬1-
-    Bool            bAutoPhrasePhrase;		//Ƿ
-    INT8            iAutoPhrase;		//Զʳ
-    Bool            bTableExactMatch;		//Ƿֻʾȷƥĺѡ/
-    Bool            bPromptTableCode;		//ϺǷʾ
+    Bool            bAutoPhrasePhrase;	//Ƿ
+    INT8            iAutoPhrase;	//Զʳ
+    Bool            bTableExactMatch;	//Ƿֻʾȷƥĺѡ/
+    Bool            bPromptTableCode;	//ϺǷʾ
 } TABLE;
 
 typedef struct _RECORD {
@@ -103,13 +105,20 @@ typedef struct _AUTOPHRASE {
 typedef union {
     AUTOPHRASE     *autoPhrase;
     RECORD         *record;
+    char            strPYPhrase[PHRASE_MAX_LENGTH * 2 + 1];
 } CANDWORD;
 
 typedef struct _TABLECANDWORD {
-    unsigned int    flag:1;	//ָʾúѡ/ԶĴʻ/
+    unsigned int    flag:2;	//ָʾúѡ/ԶĴʻ/
     CANDWORD        candWord;
 } TABLECANDWORD;
 
+typedef enum {
+    CT_NORMAL = 0,
+    CT_AUTOPHRASE,
+    CT_PYPHRASE
+} CANDTYPE;
+
 void            LoadTableInfo (void);
 Bool            LoadTableDict (void);
 void            TableInit (void);
@@ -131,13 +140,11 @@ char           *TableGetFHCandWord (int iIndex);
 Bool            HasMatchingKey (void);
 int             TableCompareCode (char *strUser, char *strDict);
 int             TableFindFirstMatchCode (void);
-RECORD         *TableFindCode (char *strHZ, Bool bMode);
 void            TableAdjustOrderByIndex (int iIndex);
 void            TableDelPhraseByIndex (int iIndex);
 void            TableDelPhraseByHZ (char *strHZ);
 void            TableDelPhrase (RECORD * record);
 RECORD         *TableHasPhrase (char *strCode, char *strHZ);
-Bool            TableCandHasPhrase (char *strHZ);
 RECORD         *TableFindPhrase (char *strHZ);
 void            TableInsertPhrase (char *strCode, char *strHZ);
 char           *TableGetCandWord (int iIndex);
