@@ -78,8 +78,9 @@ extern HOTKEYS  hkPYDelUserPhr[];
 extern HOTKEYS  hkLegend[];
 extern HOTKEYS  hkTrack[];
 extern HOTKEYS  hkGetPY[];
+extern HOTKEYS  hkGBT[];
 
-extern KEYCODE  switchKey;
+extern KEY_CODE switchKey;
 extern XIMTriggerKey *Trigger_Keys;
 extern INT8     iTriggerKeyCount;
 
@@ -107,11 +108,15 @@ extern Bool     bPhraseTips;
 extern SEMICOLON_TO_DO semicolonToDo;
 extern Bool     bEngAfterCap;
 
-/*
+//ʾٶ
 extern Bool     bShowUserSpeed;
-*/
+extern Bool     bShowVersion;
 extern Bool     bShowVK;
 
+extern char     strNameOfPinyin[];
+extern char     strNameOfShuangpin[];;
+extern char     strNameOfQuwei[];
+
 extern Bool     bFullPY;
 extern Bool     bDisablePagingInLegend;
 
@@ -136,8 +141,8 @@ extern Bool     bUseSP;
 extern Bool     bUseQW;
 extern Bool     bUseTable;
 
-extern char	strDefaultSP[];
-extern SP_FROM	iSPFrom;
+extern char     strDefaultSP[];
+extern SP_FROM  iSPFrom;
 
 //extern Bool     bLumaQQ;
 extern char     cPYYCDZ[];
@@ -154,7 +159,9 @@ extern Bool     bUseAA;
 #else
 extern char     strUserLocale[];
 #endif
+extern Bool     bUseBold;
 
+extern INT8     iOffsetX;
 extern INT8     iOffsetY;
 
 /*
@@ -189,7 +196,7 @@ void LoadConfig (Bool bMode)
     char            strPath[PATH_MAX];
     int             i;
     int             r, g, b;	//
-    Bool	    bFromUser=True;
+    Bool            bFromUser = True;
 
     strcpy (strPath, (char *) getenv ("HOME"));
     strcat (strPath, "/.fcitx/config");
@@ -202,7 +209,7 @@ void LoadConfig (Bool bMode)
     fp = fopen (strPath, "rt");
 
     if (!fp || !bFromUser)
-	SaveConfig();
+	SaveConfig ();
 
     if (!fp) {
 	LoadConfig (True);	//Ĭֵ
@@ -256,6 +263,10 @@ void LoadConfig (Bool bMode)
 	    strcpy (strUserLocale, pstr);
 	}
 #endif
+	else if (MyStrcmp (pstr, "Ƿʹô=")) {
+	    pstr += 13;
+	    bUseBold = atoi (pstr);
+	}
 	else if (MyStrcmp (pstr, "ѡʸ=")) {
 	    pstr += 11;
 	    iMaxCandWord = atoi (pstr);
@@ -295,8 +306,12 @@ void LoadConfig (Bool bMode)
 	    bShowInputWindowTriggering = atoi (pstr);
 	}
 	// ʱ
-	else if (MyStrcmp (pstr, "߶ƫ=")) {
-	    pstr += 17;
+	else if (MyStrcmp (pstr, "ƫX=")) {
+	    pstr += 14;
+	    iOffsetX = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "ƫY=")) {
+	    pstr += 14;
 	    iOffsetY = atoi (pstr);
 	}
 	//********************************************
@@ -304,10 +319,15 @@ void LoadConfig (Bool bMode)
 	    pstr += 11;
 	    bPointAfterNumber = atoi (pstr);
 	}
-	/*else if (MyStrcmp (pstr, "ʾٶ=")) {
-	   pstr += 13;
-	   bShowUserSpeed = atoi (pstr);
-	   } */
+	//ʾٶ
+	else if (MyStrcmp (pstr, "ʾٶ=")) {
+	    pstr += 13;
+	    bShowUserSpeed = atoi (pstr);
+	}
+	else if (MyStrcmp (pstr, "ʾ汾=")) {
+	    pstr += 13;
+	    bShowVersion = atoi (pstr);
+	}
 	else if (MyStrcmp (pstr, "ģʽ=")) {
 	    pstr += 15;
 	    hideMainWindow = (HIDE_MAINWINDOW) atoi (pstr);
@@ -453,6 +473,10 @@ void LoadConfig (Bool bMode)
 	    pstr += 17;
 	    SetSwitchKey (pstr);
 	}
+	else if (MyStrcmp (pstr, "GBKл=")) {
+	    pstr += 14;
+	    SetHotKey (pstr, hkGBT);
+	}
 	else if (MyStrcmp (pstr, "˫Ӣл=")) {
 	    pstr += 15;
 	    bDoubleSwitchKey = atoi (pstr);
@@ -523,16 +547,29 @@ void LoadConfig (Bool bMode)
 		i2ndSelectKey = 37;	//CTRLɨ
 		i3rdSelectKey = 109;	//CTRLɨ
 	    }
+	    else {
+		i2ndSelectKey = pstr[0];
+		i3rdSelectKey = pstr[1];
+	    }
 	}
 
 	else if (MyStrcmp (pstr, "ʹƴ=")) {
 	    pstr += 9;
 	    bUsePinyin = atoi (pstr);
 	}
+	else if (MyStrcmp (pstr, "ƴ=")) {
+	    pstr += 9;
+	    strcpy (strNameOfPinyin, pstr);
+	}
+
 	else if (MyStrcmp (pstr, "ʹ˫ƴ=")) {
 	    pstr += 9;
 	    bUseSP = atoi (pstr);
 	}
+	else if (MyStrcmp (pstr, "˫ƴ=")) {
+	    pstr += 9;
+	    strcpy (strNameOfShuangpin, pstr);
+	}
 	else if (MyStrcmp (pstr, "Ĭ˫ƴ=")) {
 	    pstr += 13;
 	    if (*pstr == ' ')
@@ -544,6 +581,10 @@ void LoadConfig (Bool bMode)
 	    pstr += 9;
 	    bUseQW = atoi (pstr);
 	}
+	else if (MyStrcmp (pstr, "λ=")) {
+	    pstr += 9;
+	    strcpy (strNameOfQuwei, pstr);
+	}
 	else if (MyStrcmp (pstr, "ʹ=")) {
 	    pstr += 9;
 	    bUseTable = atoi (pstr);
@@ -685,6 +726,7 @@ void SaveConfig (void)
 #ifdef _USE_XFT
     fprintf (fp, "ǷʹAA=%d\n", bUseAA);
 #endif
+    fprintf (fp, "Ƿʹô=%d\n", bUseBold);
 
     fprintf (fp, "\n[]\n");
     fprintf (fp, "ֺǷ=%d\n", bEngPuncAfterNumber);
@@ -706,8 +748,11 @@ void SaveConfig (void)
     fprintf (fp, "״ʾ=%d\n", bShowInputWindowTriggering);
     fprintf (fp, "#̶(뷨)0ʾ̶\n");
     fprintf (fp, "̶=%d\n", iFixedInputWindowWidth);
+    fprintf (fp, "ƫX=%d\n", iOffsetX);
+    fprintf (fp, "ƫY=%d\n", iOffsetY);
     fprintf (fp, "źӵ=%d\n", bPointAfterNumber);
-    //fprintf (fp, "ʾٶ=%d\n", bShowUserSpeed);
+    fprintf (fp, "ʾٶ=%d\n", bShowUserSpeed);
+    fprintf (fp, "ʾ汾=%d\n", bShowVersion);
 
     fprintf (fp, "ɫ=%d %d %d\n", cursorColor.color.red >> 8, cursorColor.color.green >> 8, cursorColor.color.blue >> 8);
     fprintf (fp, "ڱɫ=%d %d %d\n", mainWindowColor.backColor.red >> 8, mainWindowColor.backColor.green >> 8, mainWindowColor.backColor.blue >> 8);
@@ -739,6 +784,7 @@ void SaveConfig (void)
     fprintf (fp, "ʱ=%u\n", iTimeInterval);
     fprintf (fp, "=CTRL_K\n");
     fprintf (fp, "GBK֧=CTRL_M\n");
+    fprintf (fp, "GBKл=CTRL_ALT_F\n");
     fprintf (fp, "֧=CTRL_L\n");
     fprintf (fp, "ƴ=CTRL_ALT_E\n");
     fprintf (fp, "ȫ=SHIFT_SPACE\n");
@@ -750,7 +796,7 @@ void SaveConfig (void)
     fprintf (fp, "\n[뷨]\n");
     fprintf (fp, "ʹƴ=%d\n", bUsePinyin);
     fprintf (fp, "ʹ˫ƴ=%d\n", bUseSP);
-    fprintf (fp, "Ĭ˫ƴ=%s\n",strDefaultSP);
+    fprintf (fp, "Ĭ˫ƴ=%s\n", strDefaultSP);
     fprintf (fp, "ʹλ=%d\n", bUseQW);
     fprintf (fp, "ʹ=%d\n", bUseTable);
     fprintf (fp, "ʾʿеĴ=%d\n", bPhraseTips);
@@ -1165,8 +1211,8 @@ char           *ConvertGBKSimple2Tradition (char *strHZ)
 		       * (unsigned char) 0xbe + ((unsigned char) strHZ[i + 1] - (unsigned char) 0x40)
 		       - ((unsigned char) strHZ[i + 1] / (unsigned char) 0x80)) * 2;
 		if (idx >= 0 && idx < gGBKS2TTableSize - 1) {
-		    if (	/*(unsigned char)gGBKS2TTable[idx] != (unsigned char)0xa1 
-				   && */ (unsigned char) gGBKS2TTable[idx + 1] != (unsigned char) 0x7f) {
+		    //if ((unsigned char)gGBKS2TTable[idx] != (unsigned char)0xa1 && (unsigned char) gGBKS2TTable[idx + 1] != (unsigned char) 0x7f) {
+		    if ((unsigned char) gGBKS2TTable[idx + 1] != (unsigned char) 0x7f) {
 			ret[i] = gGBKS2TTable[idx];
 			ret[i + 1] = gGBKS2TTable[idx + 1];
 			i += 1;
@@ -1180,3 +1226,8 @@ char           *ConvertGBKSimple2Tradition (char *strHZ)
 
     return ret;
 }
+
+int CalHZIndex (char *strHZ)
+{
+    return (strHZ[0] + 127) * 255 + strHZ[1] + 128;
+}
