@@ -73,7 +73,7 @@ int             StringWidth (char *str, XFontSet font);
 int             FontHeight (XFontSet font);
 #endif
 
-int             FillImageByXPMData (XImage * pImage, char **apcData);
+/*int             FillImageByXPMData (XImage * pImage, char **apcData);*/
 Bool            MouseClick (int *x, int *y, Window window);
 /*
 void            WaitButtonRelease (XPoint * point);
